;==================================================================
;
; Author:         Freak
; Date:           September 20, 2014
; Explain:
;           
; Typical Usage:  A sample use of JSON to store and track user information. This method allows future
;                       changes to be made in a structure and still keep existing data.


;Reading and writing individual values in a binary file is very cumbersome and it is very easy 
;to make mistakes. It is also not very flexible. For example, it will be quite some work to add 
;a new field to the PLAYER_STATS structure because all code needs to be updated to properly 
;handle it.
;
;I instead suggest to write the data in a serialization format such as XML or JSON. PureBasic 
;provides libraries to deal with such formats in a simple way. The latest PureBasic version 
; (5.30) provides commands to easily transfer data between an Array or Linked list and these 
;serialization format without the need to worry about the details.
;
;I have included an example below. Some points about the example:
;
;I use a LinkedList in the below example rather than an Array to hold the player data because 
;it is much easier to reorganize (add, remove elements etc) than with an array. The 
;reading/writing part would work just as well with an array though.

;The idea is that the PLAYERS() list always holds information about all players. If you need to 
;change something, you change the information in the list and then re-write the entire 
;players file to disk. This is much easier than trying to update an individual player in the file on disk.

;Notice how short the code to read and write the file is. There is no need to handle the 
;individual structure fields yourself. PureBasic does this for you.

;This code is much easier to change in the future. If you add a new field to the PLAYER_STATS 
;structure, there is no need to modify the load/save code at all. In fact, if you add a new field 
;to the structure, you can still read a players file that was written with the old version 
;(without the field). The new field will simply be empty if you read back such an old file. This 
;makes it very flexible for future changes.

;==================================================================
Structure PLAYER_STATS
  PLAYER$
  CREDITS.i
  COST.i
  DATE$
  TIME$
EndStructure

NewList PLAYERS.PLAYER_STATS()

; ------- Example data ---------
AddElement(PLAYERS())
     PLAYERS()\PLAYER$ = "Player 1"
     PLAYERS()\CREDITS = 999
     PLAYERS()\COST = 10
     PLAYERS()\DATE$ = FormatDate("%mm-%dd-%yyyy", Date())
     PLAYERS()\TIME$ = FormatDate("%hh:%II:%ss", Date())

AddElement(PLAYERS())
     PLAYERS()\PLAYER$ = "Player 2"
     PLAYERS()\CREDITS = 123
     PLAYERS()\COST = 456
     PLAYERS()\DATE$ = FormatDate("%mm-%dd-%yyyy", Date())
     PLAYERS()\TIME$ = FormatDate("%hh:%II:%ss", Date())

AddElement(PLAYERS())
     PLAYERS()\PLAYER$ = "Player 3"
     PLAYERS()\CREDITS = 0
     PLAYERS()\COST = 1000000
     PLAYERS()\DATE$ = FormatDate("%mm-%dd-%yyyy", Date())
     PLAYERS()\TIME$ = FormatDate("%hh:%II:%ss", Date())




; ------- Write file to disk ---------
If CreateJSON(0)                                       ; Create JSON object for output
  InsertJSONList(JSONValue(0), PLAYERS())              ; Fill JSON object with the player list
  SaveJSON(0, "PLAYERS.DAT", #PB_JSON_PrettyPrint)     ; Write JSON object to disk in a readable (pretty-printed) format
  FreeJSON(0)                                          ; Free JSON object
EndIf
; -----------------------------------




; ------- Load file from disk ---------
If LoadJSON(0, "PLAYERS.DAT")                          ; Load the JSON object from the file
  ExtractJSONList(JSONValue(0), PLAYERS())             ; Read the player data from JSON into the player list
  FreeJSON(0)                                          ; Free JSON object
Else
  Debug "Error reading file: " + JSONErrorMessage()    ; File cannot be read for some reason
EndIf
; -----------------------------------




; ------- Show loaded players ---------
ForEach PLAYERS()
  Debug "Name:   " + PLAYERS()\PLAYER$
  Debug "Credits: " + PLAYERS()\CREDITS
  Debug "Cost:    " + PLAYERS()\COST
  Debug "Date:    " + PLAYERS()\DATE$
  Debug "Time:    " + PLAYERS()\TIME$
  Debug "-----"
Next PLAYERS()
; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 104
; EnableUnicode
; EnableXP
; Executable = JSON Sample.exe